package com.citrusengine.objects.platformer {

	import com.citrusengine.objects.PhysicsObject;

	import org.osflash.signals.Signal;

	import flash.events.TimerEvent;
	import flash.utils.Timer;


	/**
	 * @author Aymeric
	 */
	public class Cannon extends Platform {

		[Inspectable(defaultValue=2000)]
		public var fireRate:Number = 2000;

		[Inspectable(defaultValue="right",enumeration="right,left")]
		public var startingDirection:String = "right";

		[Inspectable(defaultValue=20)]
		public var missileWidth:uint = 20;

		[Inspectable(defaultValue=20)]
		public var missileHeight:uint = 20;

		[Inspectable(defaultValue=2)]
		public var missileSpeed:Number = 2;

		[Inspectable(defaultValue=0)]
		public var missileAngle:Number = 0;

		[Inspectable(defaultValue=1000)]
		public var missileExplodeDuration:Number = 1000;

		[Inspectable(defaultValue=10000)]
		public var missileFuseDuration:Number = 10000;

		[Inspectable(defaultValue="",format="File")]
		public var missileView:String = "";
		
		[Inspectable(defaultValue=true)]
		public var openFire:Boolean = true;

		public var onGiveDamage:Signal;

		protected var _firing:Boolean = false;

		protected var _timer:Timer;

		public function Cannon(name:String, params:Object = null) {
			
			super(name, params);

			onGiveDamage = new Signal(PhysicsObject);
			
			if (openFire)
				startFire();
		}

		override public function destroy():void {

			onGiveDamage.removeAll();

			_timer.stop();
			_timer.removeEventListener(TimerEvent.TIMER, _fire);

			super.destroy();
		}

		override public function update(timeDelta:Number):void {

			super.update(timeDelta);

			_updateAnimation();
		}

		protected function _damage(missile:Missile, contact:PhysicsObject):void {

			if (contact != null) {
				onGiveDamage.dispatch(contact);
			}
		}

		public function startFire():void {

			_firing = true;

			_timer = new Timer(fireRate);
			_timer.addEventListener(TimerEvent.TIMER, _fire);
			_timer.start();
		}

		public function stopFire():void {

			_firing = false;

			_timer.stop();
			_timer.removeEventListener(TimerEvent.TIMER, _fire);
		}

		protected function _fire(tEvt:TimerEvent):void {

			var missile:Missile;

			if (startingDirection == "right") {
				missile = new Missile("Missile", {x:x + width, y:y, width:missileWidth, height:missileHeight, speed:missileSpeed, angle:missileAngle, explodeDuration:missileExplodeDuration, fuseDuration:missileFuseDuration, view:missileView});
			} else {
				missile = new Missile("Missile", {x:x - width, y:y, width:missileWidth, height:missileHeight, speed:-missileSpeed, angle:missileAngle, explodeDuration:missileExplodeDuration, fuseDuration:missileFuseDuration, view:missileView});
			}

			_ce.state.add(missile);
			missile.onExplode.addOnce(_damage);
		}

		protected function _updateAnimation():void {

			if (_firing) {
				_animation = "fire";
			} else {
				_animation = "normal";
			}
		}
	}
}